@@ -15,9 +15,11 @@
 from django.utils.translation import ugettext_lazy as _
 
 from oslo_log import log as logging
+import six
 
 from muranoclient.common import exceptions as exc
 from muranodashboard import api
+from muranodashboard.api import packages as packages_api
 from muranodashboard.common import utils
 from muranodashboard.environments import consts
 from muranodashboard.environments import topology
@@ -58,7 +60,7 @@ class Session(object):
         """Get an open session id
 
         Gets id from already opened session for specified environment,
-        otherwise opens new session and returns it's id
+        otherwise opens new session and returns its id
 
         :param request:
         :param environment_id:
@@ -70,23 +72,22 @@ class Session(object):
         if environment_id in sessions:
             id = sessions[environment_id]
         else:
-            id = create_session(environment_id, request)
+            id = create_session(request, environment_id)
         return id
 
     @staticmethod
     def get_or_create_or_delete(request, environment_id):
         """Get an open session id
 
-        Gets id from session in open state for specified environment,
-        if state is deployed - this session will be deleted and new
-        would be created. If there are no any sessions new would be created.
-        Returns if of chosen or created session.
+        Gets id from session in open state for specified environment.
+        If state is deployed, then the session is deleted and a new one
+        is created. If there are no sessions, then a new one is created.
+        Returns id of chosen or created session.
 
         :param request:
         :param environment_id:
-        :return: Session Id
+        :return: Session id
         """
-
         sessions = request.session.get('sessions', {})
         client = api.muranoclient(request)
 
@@ -96,7 +97,7 @@ class Session(object):
                 session_data = client.sessions.get(environment_id, id)
             except exc.HTTPForbidden:
                 del sessions[environment_id]
-                LOG.debug("The environment is being deployed by other user."
+                LOG.debug("The environment is being deployed by other user. "
                           "Creating a new session "
                           "for the environment {0}".format(environment_id))
                 return create_session(request, environment_id)
@@ -111,10 +112,31 @@ class Session(object):
         else:
             LOG.debug("Creating a new session")
             return create_session(request, environment_id)
-        LOG.debug("Found active session "
-                  "for the environment {0}".format(environment_id))
+        LOG.debug("Found active session for the environment {0}"
+                  .format(environment_id))
         return id
 
+    @staticmethod
+    def get_if_available(request, environment_id):
+        """Get an id of open session if it exists and is not in deployed state.
+
+        Returns None otherwise
+        """
+        sessions = request.session.get('sessions', {})
+        client = api.muranoclient(request)
+
+        if environment_id in sessions:
+            id = sessions[environment_id]
+            try:
+                session_data = client.sessions.get(environment_id, id)
+            except exc.HTTPForbidden:
+                return None
+            else:
+                if session_data.state in [consts.STATUS_ID_DEPLOY_FAILURE,
+                                          consts.STATUS_ID_READY]:
+                    return None
+                return id
+
     @staticmethod
     def get(request, environment_id):
         """Get an open session id
@@ -153,12 +175,25 @@ class Session(object):
         request.session['sessions'] = sessions
 
 
-def _update_env(env):
-    env.has_new_services = False
+def _update_env(env, request):
+    # TODO(vakovalchuk): optimize latest deployment when limit is available
+    deployments = deployments_list(request, env.id)
+    if deployments:
+        latest_deployment = deployments[0]
+        deployed_services = {service['?']['id'] for service in
+                             latest_deployment.description['services']}
+    else:
+        deployed_services = set()
+
     if env.services:
-        for service in env.services:
-            if service['?']['status'] == 'pending':
-                env.has_new_services = True
+        current_services = {service['?']['id'] for service in env.services}
+    else:
+        current_services = set()
+
+    env.has_new_services = current_services != deployed_services
+
+    if not env.has_new_services and env.status == consts.STATUS_ID_PENDING:
+        env.status = consts.STATUS_ID_READY
 
     if not env.has_new_services and env.version == 0:
         if env.status == consts.STATUS_ID_READY:
@@ -206,7 +241,7 @@ def environment_get(request, environment_id):
         env = client.environments.get(environment_id, acquired)
         Session.set(request, environment_id, acquired)
 
-    env = _update_env(env)
+    env = _update_env(env, request)
 
     LOG.debug('Environment::Get {0}'.format(env))
     return env
@@ -242,7 +277,7 @@ def services_list(request, environment_id):
         return u'%s...' % msg[:to] if len(msg) > to else msg
 
     services = []
-    # need to create new session to see services deployed be other user
+    # need to create new session to see services deployed by other user
     session_id = Session.get(request, environment_id)
 
     get_environment = api.muranoclient(request).environments.get
@@ -276,6 +311,18 @@ def services_list(request, environment_id):
         service_data['operation_updated'] = time
         if service_data['?'].get('name'):
             service_data['name'] = service_data['?']['name']
+        if (consts.DASHBOARD_ATTRS_KEY not in service_data['?'] or
+                not service_data['?'][consts.DASHBOARD_ATTRS_KEY].get('name')):
+            fqn = service_data['?']['type']
+            version = None
+            if '/' in fqn:
+                version, fqn = fqn.split('/')[1].split('@')
+            pkg = packages_api.app_by_fqn(request, fqn, version=version)
+            if pkg:
+                app_name = pkg.name
+                storage = service_data['?'].setdefault(
+                    consts.DASHBOARD_ATTRS_KEY, {})
+                storage['name'] = app_name
 
         services.append(service_data)
 
@@ -288,12 +335,13 @@ def service_list_by_fqns(request, environment_id, fqns):
         return []
     services = services_list(request, environment_id)
     LOG.debug('Service::Instances::List')
-    return [service for service in services if service['?']['type'] in fqns]
+    return [service for service in services
+            if service['?']['type'].split('/')[0] in fqns]
 
 
 def service_create(request, environment_id, parameters):
-    # we should be able to delete session
-    # if we what add new services to this environment
+    # We should be able to delete session if we want to add new services to
+    # this environment.
     session_id = Session.get_or_create_or_delete(request, environment_id)
     LOG.debug('Service::Create {0}'.format(parameters['?']['type']))
     return api.muranoclient(request).services.post(environment_id,
@@ -325,7 +373,7 @@ def extract_actions_list(service):
         return dict(_action.items() + [('id', action_id)])
 
     return [make_action_datum(_id, action) for (_id, action) in
-            actions_data.iteritems() if action.get('enabled')]
+            six.iteritems(actions_data) if action.get('enabled')]
 
 
 def run_action(request, environment_id, action_id):
