@@ -27,16 +27,17 @@ from horizon import messages
 from horizon import tables
 from horizon.utils import filters
 from muranoclient.common import exceptions as exc
+from openstack_dashboard import policy
 from oslo_log import log as logging
 
 from muranodashboard import api as api_utils
 from muranodashboard.api import packages as pkg_api
 from muranodashboard.catalog import views as catalog_views
+from muranodashboard.common import utils as md_utils
 from muranodashboard.environments import api
 from muranodashboard.environments import consts
 from muranodashboard.packages import consts as pkg_consts
 
-
 LOG = logging.getLogger(__name__)
 
 
@@ -48,6 +49,25 @@ def _get_environment_status_and_version(request, table):
     return status, version
 
 
+def _check_row_actions_allowed(action, request):
+    envs = action.table.data
+    if not envs:
+        return False
+    for env in envs:
+        if action.allowed(request, env):
+            return True
+    return False
+
+
+def _environment_has_deployed_services(request, environment_id):
+    deployments = api.deployments_list(request, environment_id)
+    if not deployments:
+        return False
+    if not deployments[0].description['services']:
+        return False
+    return True
+
+
 class AddApplication(tables.LinkAction):
     name = 'AddApplication'
     verbose_name = _('Add Component')
@@ -59,19 +79,20 @@ class AddApplication(tables.LinkAction):
         return status not in consts.NO_ACTION_ALLOWED_STATUSES
 
     def get_link_url(self, datum=None):
-        base_url = reverse('horizon:murano:catalog:switch_env',
+        base_url = reverse('horizon:app-catalog:catalog:switch_env',
                            args=(self.table.kwargs['environment_id'],))
-        redirect_url = reverse('horizon:murano:catalog:index')
+        redirect_url = reverse('horizon:app-catalog:catalog:index')
         return '{0}?next={1}'.format(base_url, redirect_url)
 
 
 class CreateEnvironment(tables.LinkAction):
     name = 'CreateEnvironment'
     verbose_name = _('Create Environment')
-    url = 'horizon:murano:environments:create_environment'
+    url = 'horizon:app-catalog:environments:create_environment'
     classes = ('btn-launch', 'add_env')
-    redirect_url = "horizon:project:murano:environments"
+    redirect_url = "horizon:app-catalog:environments:index"
     icon = 'plus'
+    policy_rules = (("murano", "create_environment"),)
 
     def allowed(self, request, datum):
         return True if self.table.data else False
@@ -87,8 +108,9 @@ class CreateEnvironment(tables.LinkAction):
             exceptions.handle(request, msg, redirect=redirect)
 
 
-class DeleteEnvironment(tables.DeleteAction):
-    redirect_url = "horizon:project:murano:environments"
+class DeleteEnvironment(policy.PolicyTargetMixin, tables.DeleteAction):
+    redirect_url = "horizon:app-catalog:environments:index"
+    policy_rules = (("murano", "delete_environment"),)
 
     @staticmethod
     def action_present(count):
@@ -101,16 +123,19 @@ class DeleteEnvironment(tables.DeleteAction):
     @staticmethod
     def action_past(count):
         return ungettext_lazy(
-            u"Start Deleting Environment",
-            u"Start Deleting Environments",
+            u"Started Deleting Environment",
+            u"Started Deleting Environments",
             count
         )
 
     def allowed(self, request, environment):
-        if environment:
-            return environment.status not in (consts.STATUS_ID_DEPLOYING,
-                                              consts.STATUS_ID_DELETING)
-        return True
+        # table action case: action allowed if any row action allowed
+        if not environment:
+            return _check_row_actions_allowed(self, request)
+
+        # row action case
+        return environment.status not in (consts.STATUS_ID_DEPLOYING,
+                                          consts.STATUS_ID_DELETING)
 
     def action(self, request, environment_id):
         try:
@@ -127,7 +152,8 @@ class AbandonEnvironment(tables.DeleteAction):
     help_text = _("This action cannot be undone. Any resources created by "
                   "this environment will have to be released manually.")
     name = 'abandon'
-    redirect_url = "horizon:project:murano:environments"
+    redirect_url = "horizon:app-catalog:environments:index"
+    policy_rules = (("murano", "delete_environment"),)
 
     @staticmethod
     def action_present(count):
@@ -152,6 +178,12 @@ class AbandonEnvironment(tables.DeleteAction):
          * environment is new
          * app added to env, but not deploy is not started
         """
+
+        # table action case: action allowed if any row action allowed
+        if not environment:
+            return _check_row_actions_allowed(self, request)
+
+        # row action case
         status = getattr(environment, 'status', None)
         if status in [consts.STATUS_ID_NEW, consts.STATUS_ID_PENDING]:
             return False
@@ -181,8 +213,8 @@ class DeleteService(tables.DeleteAction):
     @staticmethod
     def action_past(count):
         return ungettext_lazy(
-            u"Start Deleting Component",
-            u"Start Deleting Components",
+            u"Started Deleting Component",
+            u"Started Deleting Components",
             count
         )
 
@@ -201,16 +233,17 @@ class DeleteService(tables.DeleteAction):
                                        service_id)
         except Exception:
             msg = _('Sorry, you can\'t delete service right now')
-            redirect = reverse("horizon:murano:environments:index")
+            redirect = reverse("horizon:app-catalog:environments:index")
             exceptions.handle(request, msg, redirect=redirect)
 
 
 class DeployEnvironment(tables.BatchAction):
     name = 'deploy'
     classes = ('btn-launch',)
+    icon = "play"
 
     @staticmethod
-    def action_present(count):
+    def action_present_deploy(count):
         return ungettext_lazy(
             u"Deploy Environment",
             u"Deploy Environments",
@@ -218,37 +251,67 @@ class DeployEnvironment(tables.BatchAction):
         )
 
     @staticmethod
-    def action_past(count):
+    def action_past_deploy(count):
         return ungettext_lazy(
-            u"Deployed Environment",
+            u"Started deploying Environment",
+            u"Started deploying Environments",
+            count
+        )
+
+    @staticmethod
+    def action_present_update(count):
+        return ungettext_lazy(
+            u"Update Environment",
+            # there can be cases when some of the envs are new and some are not
+            # so it is better to just leave "Deploy" for multiple envs
+            u"Deploy Environments",
+            count
+        )
+
+    @staticmethod
+    def action_past_update(count):
+        return ungettext_lazy(
+            u"Updated Environment",
             u"Deployed Environments",
             count
         )
 
+    action_present = action_present_deploy
+    action_past = action_past_deploy
+
     def allowed(self, request, environment):
         """Limit when 'Deploy Environment' button is shown
 
-        'Deploy environment' is not shown in several cases:
-        * when deploy is already in progress
-        * delete is in progress
-        * no new services added to the environment (after env creation
-          or successful deploy or delete failure)
+        'Deploy environment' is shown when set of environment's services
+        changed or previous deploy failed.
+        If environment has already deployed services,
+        button is shown as 'Update environment'
         """
+
+        # table action case: action allowed if any row action allowed
+        if not environment:
+            return _check_row_actions_allowed(self, request)
+
+        # row action case
+        if _environment_has_deployed_services(request, environment.id):
+            self.action_present = self.action_present_update
+            self.action_past = self.action_past_update
+        else:
+            self.action_present = self.action_present_deploy
+            self.action_past = self.action_past_deploy
+
         status = getattr(environment, 'status', None)
-        if (status != consts.STATUS_ID_DEPLOY_FAILURE and
-                not environment.has_new_services):
-            return False
-        if (status in consts.NO_ACTION_ALLOWED_STATUSES or
-                status == consts.STATUS_ID_READY):
-            return False
-        return True
+        if status in (consts.STATUS_ID_PENDING,
+                      consts.STATUS_ID_DEPLOY_FAILURE):
+            return True
+        return False
 
     def action(self, request, environment_id):
         try:
             api.environment_deploy(request, environment_id)
         except Exception:
             msg = _('Unable to deploy. Try again later')
-            redirect = reverse('horizon:murano:environments:index')
+            redirect = reverse('horizon:app-catalog:environments:index')
             exceptions.handle(request, msg, redirect=redirect)
 
 
@@ -257,16 +320,25 @@ class DeployThisEnvironment(tables.Action):
     verbose_name = _('Deploy This Environment')
     requires_input = False
     classes = ('btn-launch',)
+    icon = "play"
 
     def allowed(self, request, service):
-        """Limit when 'Deploy Environment' button is shown
+        """Limit when 'Deploy This Environment' button is shown
 
         'Deploy environment' is not shown in several cases:
         * when deploy is already in progress
         * delete is in progress
         * env was just created and no apps added
         * previous deployment finished successfully
+        If environment has already deployed services, button is shown
+        as 'Update This Environment'
         """
+        environment_id = self.table.kwargs['environment_id']
+        if _environment_has_deployed_services(request, environment_id):
+            self.verbose_name = _('Update This Environment')
+        else:
+            self.verbose_name = _('Deploy This Environment')
+
         status, version = _get_environment_status_and_version(request,
                                                               self.table)
         if (status in consts.NO_ACTION_ALLOWED_STATUSES or
@@ -287,16 +359,16 @@ class DeployThisEnvironment(tables.Action):
             msg = _('Unable to deploy. Try again later')
             exceptions.handle(
                 request, msg,
-                redirect=reverse('horizon:murano:environments:index'))
+                redirect=reverse('horizon:app-catalog:environments:index'))
         return shortcuts.redirect(
-            reverse('horizon:murano:environments:services',
+            reverse('horizon:app-catalog:environments:services',
                     args=(environment_id,)))
 
 
 class ShowEnvironmentServices(tables.LinkAction):
     name = 'show'
     verbose_name = _('Manage Components')
-    url = 'horizon:murano:environments:services'
+    url = 'horizon:app-catalog:environments:services'
 
     def allowed(self, request, environment):
         return True
@@ -305,6 +377,11 @@ class ShowEnvironmentServices(tables.LinkAction):
 class UpdateEnvironmentRow(tables.Row):
     ajax = True
 
+    def __init__(self, table, datum=None):
+        super(UpdateEnvironmentRow, self).__init__(table, datum)
+        if hasattr(datum, 'status'):
+            self.attrs['status'] = datum.status
+
     def get_data(self, request, environment_id):
         try:
             return api.environment_get(request, environment_id)
@@ -327,6 +404,10 @@ class UpdateServiceRow(tables.Row):
 
 
 class UpdateName(tables.UpdateAction):
+    def allowed(self, request, environment, cell):
+        policy_rule = (("murano", "update_environment"),)
+        return policy.check(policy_rule, request)
+
     def update_cell(self, request, datum, obj_id, cell_name, new_cell_value):
         try:
             if not new_cell_value or new_cell_value.isspace():
@@ -336,9 +417,10 @@ class UpdateName(tables.UpdateAction):
             mc = api_utils.muranoclient(request)
             mc.environments.update(datum.id, name=new_cell_value)
         except exc.HTTPConflict:
-            message = _("This name is already taken.")
+            message = _("Couldn't update environment. Reason: This name is "
+                        "already taken.")
             messages.warning(request, message)
-            LOG.warning(_("Couldn't update environment. Reason: ") + message)
+            LOG.warning(message)
 
             # FIXME(kzaitsev): There is a bug in horizon and inline error
             # icons are missing. This means, that if we return 400 here, by
@@ -354,13 +436,44 @@ class UpdateName(tables.UpdateAction):
         return True
 
 
+class UpdateEnvMetadata(tables.LinkAction):
+    name = "update_env_metadata"
+    verbose_name = _("Update Metadata")
+    ajax = False
+    icon = "pencil"
+    attrs = {"ng-controller": "MetadataModalHelperController as modal"}
+
+    def __init__(self, attrs=None, **kwargs):
+        kwargs['preempt'] = True
+        self.session_id = None
+        super(UpdateEnvMetadata, self).__init__(attrs, **kwargs)
+
+    def get_link_url(self, environment):
+        target = json.dumps({
+            'environment': environment.id,
+            'session': self.session_id
+        })
+        self.attrs['ng-click'] = (
+            "modal.openMetadataModal('muranoenv', %s, true)" % target)
+        return "javascript:void(0);"
+
+    def allowed(self, request, environment=None):
+        return environment.status not in (consts.STATUS_ID_DEPLOYING,
+                                          consts.STATUS_ID_DELETING)
+
+    def update(self, request, datum):
+        if datum:
+            env_id = datum.id
+            self.session_id = api.Session.get_if_available(request, env_id)
+
+
 class EnvironmentsTable(tables.DataTable):
-    name = tables.Column('name',
-                         link='horizon:murano:environments:services',
-                         verbose_name=_('Name'),
-                         form_field=forms.CharField(required=False),
-                         update_action=UpdateName,
-                         truncate=40)
+    name = md_utils.Column(
+        'name',
+        link='horizon:app-catalog:environments:services',
+        verbose_name=_('Name'),
+        form_field=forms.CharField(required=False),
+        update_action=UpdateName)
 
     status = tables.Column('status',
                            verbose_name=_('Status'),
@@ -368,6 +481,21 @@ class EnvironmentsTable(tables.DataTable):
                            status_choices=consts.STATUS_CHOICES,
                            display_choices=consts.STATUS_DISPLAY_CHOICES)
 
+    def get_env_detail_link(self, environment):
+        # NOTE: using the policy check for show_environment
+        if policy.check((("murano", "show_environment"),),
+                        self.request, target={"environment": environment}):
+            return reverse("horizon:app-catalog:environments:services",
+                           args=(environment.id,))
+        return None
+
+    def __init__(self, request, data=None, needs_form_wrapper=None, **kwargs):
+        super(EnvironmentsTable,
+              self).__init__(request, data=data,
+                             needs_form_wrapper=needs_form_wrapper,
+                             **kwargs)
+        self.columns['name'].get_link_url = self.get_env_detail_link
+
     class Meta(object):
         name = 'environments'
         verbose_name = _('Environments')
@@ -375,14 +503,15 @@ class EnvironmentsTable(tables.DataTable):
         row_class = UpdateEnvironmentRow
         status_columns = ['status']
         no_data_message = _('NO ENVIRONMENTS')
-        table_actions = (CreateEnvironment,)
+        table_actions = (CreateEnvironment, DeployEnvironment,
+                         DeleteEnvironment, AbandonEnvironment)
         row_actions = (ShowEnvironmentServices, DeployEnvironment,
-                       DeleteEnvironment, AbandonEnvironment)
-        multi_select = False
+                       DeleteEnvironment, AbandonEnvironment,
+                       UpdateEnvMetadata)
 
 
 def get_service_details_link(service):
-    return reverse('horizon:murano:environments:service_details',
+    return reverse('horizon:app-catalog:environments:service_details',
                    args=(service.environment_id, service['?']['id']))
 
 
@@ -390,10 +519,45 @@ def get_service_type(datum):
     return datum['?'].get(consts.DASHBOARD_ATTRS_KEY, {}).get('name')
 
 
+class UpdateMetadata(tables.LinkAction):
+    name = "update_metadata"
+    verbose_name = _("Update Metadata")
+    ajax = False
+    icon = "pencil"
+    attrs = {"ng-controller": "MetadataModalHelperController as modal"}
+
+    def __init__(self, attrs=None, **kwargs):
+        kwargs['preempt'] = True
+        self.session_id = None
+        super(UpdateMetadata, self).__init__(attrs, **kwargs)
+
+    def get_link_url(self, service):
+        env_id = self.table.kwargs.get('environment_id')
+        comp_id = service['?']['id']
+        target = json.dumps({
+            'environment': env_id,
+            'component': comp_id,
+            'session': self.session_id,
+        })
+        self.attrs['ng-click'] = (
+            "modal.openMetadataModal('muranoapp', %s, true)" % target)
+        return "javascript:void(0);"
+
+    def allowed(self, request, service=None):
+        status, version = _get_environment_status_and_version(request,
+                                                              self.table)
+        return status != consts.STATUS_ID_DEPLOYING
+
+    def update(self, request, datum):
+        env_id = self.table.kwargs.get('environment_id')
+        self.session_id = api.Session.get_if_available(request, env_id)
+
+
 class ServicesTable(tables.DataTable):
-    name = tables.Column('name',
-                         verbose_name=_('Name'),
-                         link=get_service_details_link)
+    name = md_utils.Column(
+        'name',
+        verbose_name=_('Name'),
+        link=get_service_details_link)
 
     _type = tables.Column(get_service_type,
                           verbose_name=_('Type'))
@@ -419,7 +583,7 @@ class ServicesTable(tables.DataTable):
             packages, self._more = pkg_api.package_list(
                 self.request,
                 filters={'type': 'Application', 'catalog': True})
-        return json.dumps([package.to_dict() for package in packages])
+        return [package.to_dict() for package in packages]
 
     def actions_allowed(self):
         status, version = _get_environment_status_and_version(
@@ -438,8 +602,8 @@ class ServicesTable(tables.DataTable):
 
             class CustomAction(tables.LinkAction):
                 name = action_datum['name']
-                verbose_name = action_datum['name']
-                url = reverse('horizon:murano:environments:start_action',
+                verbose_name = action_datum.get('title') or name
+                url = reverse('horizon:app-catalog:environments:start_action',
                               args=(environment_id, action_datum['id']))
                 classes = _classes
                 table = self
@@ -465,10 +629,10 @@ class ServicesTable(tables.DataTable):
         return actions
 
     def get_repo_url(self):
-        return pkg_consts.MURANO_REPO_URL
+        return pkg_consts.DISPLAY_MURANO_REPO_URL
 
     def get_pkg_def_url(self):
-        return reverse('horizon:murano:packages:index')
+        return reverse('horizon:app-catalog:packages:index')
 
     class Meta(object):
         name = 'services'
@@ -477,7 +641,7 @@ class ServicesTable(tables.DataTable):
         status_columns = ['status']
         row_class = UpdateServiceRow
         table_actions = (AddApplication, DeployThisEnvironment)
-        row_actions = (DeleteService,)
+        row_actions = (DeleteService, UpdateMetadata)
         multi_select = False
 
 
@@ -488,7 +652,7 @@ class ShowDeploymentDetails(tables.LinkAction):
     def get_link_url(self, deployment=None):
         kwargs = {'environment_id': deployment.environment_id,
                   'deployment_id': deployment.id}
-        return reverse('horizon:murano:environments:deployment_details',
+        return reverse('horizon:app-catalog:environments:deployment_details',
                        kwargs=kwargs)
 
     def allowed(self, request, environment):
@@ -516,8 +680,7 @@ class DeploymentsTable(tables.DataTable):
 
 
 class EnvConfigTable(tables.DataTable):
-    name = tables.Column('name',
-                         verbose_name=_('Name'))
+    name = md_utils.Column('name', verbose_name=_('Name'))
     _type = tables.Column(
         lambda datum: get_service_type(datum) or 'Unknown',
         verbose_name=_('Type'))
