@@ -12,7 +12,7 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-from collections import OrderedDict
+import collections
 import json
 
 from django.conf import settings
@@ -22,6 +22,7 @@ from horizon import exceptions
 from horizon import tabs
 from openstack_dashboard.api import heat as heat_api
 from openstack_dashboard.api import nova as nova_api
+from openstack_dashboard import policy
 
 from muranoclient.common import exceptions as exc
 from muranodashboard.environments import api
@@ -40,26 +41,28 @@ class OverviewTab(tabs.Tab):
         :param request:
         :return:
         """
-        def find_stack(name):
-            stacks, has_more, has_prev = heat_api.stacks_list(request)
+        def find_stack(name, **kwargs):
+            stacks, has_more, has_prev = heat_api.stacks_list(
+                request, sort_dir='asc', **kwargs)
             for stack in stacks:
                 if name in stack.stack_name:
                     stack_data = {'id': stack.id,
                                   'name': stack.stack_name}
                     return stack_data
             if has_more:
-                find_stack()
+                return find_stack(name, marker=stacks[-1].id)
             return {}
 
         def get_instance_and_stack(instance_data, request):
             instance_name = instance_data['name']
+            nova_openstackid = instance_data['openstackId']
             stack_name = ''
             instance_result_data = {}
             stack_result_data = {}
             instances, more = nova_api.server_list(request)
 
             for instance in instances:
-                if instance_name in instance.name:
+                if nova_openstackid in instance.id:
                     instance_result_data = {'name': instance.name,
                                             'id': instance.id}
                     stack_name = instance.name.split('-' + instance_name)[0]
@@ -76,7 +79,7 @@ class OverviewTab(tabs.Tab):
             if id == service_data['?']['status']:
                 status_name = name
 
-        detail_info = OrderedDict([
+        detail_info = collections.OrderedDict([
             ('Name', getattr(service_data, 'name', '')),
             ('ID', service_data['?']['id']),
             ('Type', tables.get_service_type(service_data) or 'Unknown'),
@@ -109,7 +112,7 @@ class OverviewTab(tabs.Tab):
                     detail_info['Stack'] = stack
 
         if hasattr(service_data,
-                   'instances') and service_data['instance'] is not None:
+                   'instances') and service_data['instances'] is not None:
                 instances_for_template = []
                 stacks_for_template = []
                 for instance in service_data['instances']:
@@ -202,7 +205,7 @@ class EnvironmentServicesTab(tabs.TableTab):
     def get_services_data(self):
         services = []
         self.environment_id = self.tab_group.kwargs['environment_id']
-        ns_url = "horizon:murano:environments:index"
+        ns_url = "horizon:app-catalog:environments:index"
         try:
             services = api.services_list(self.request, self.environment_id)
         except exc.HTTPForbidden:
@@ -235,10 +238,13 @@ class DeploymentTab(tabs.TableTab):
     template_name = 'horizon/common/_detail_table.html'
     preload = False
 
+    def allowed(self, request):
+        return policy.check((("murano", "list_deployments"),), request)
+
     def get_deployments_data(self):
         deployments = []
         self.environment_id = self.tab_group.kwargs['environment_id']
-        ns_url = "horizon:murano:environments:index"
+        ns_url = "horizon:app-catalog:environments:index"
         try:
             deployments = api.deployments_list(self.request,
                                                self.environment_id)
@@ -259,13 +265,16 @@ class EnvironmentDetailsTabs(tabs.TabGroup):
     slug = "environment_details"
     tabs = (EnvironmentServicesTab, EnvironmentTopologyTab,
             DeploymentTab, LatestLogsTab)
+    sticky = True
 
 
 class ServicesTabs(tabs.TabGroup):
     slug = "services_details"
     tabs = (OverviewTab, ServiceLogsTab)
+    sticky = True
 
 
 class DeploymentDetailsTabs(tabs.TabGroup):
     slug = "deployment_details"
     tabs = (EnvConfigTab, EnvLogsTab,)
+    sticky = True
