@@ -12,14 +12,16 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-import logging
 import json
+
+from django.core.urlresolvers import reverse
 from django import forms
 from django.utils.translation import ugettext_lazy as _
-from django.forms import ValidationError
-from horizon.forms import SelfHandlingForm
-from horizon import messages, exceptions
+from horizon import exceptions
+from horizon import forms as horizon_forms
+from horizon import messages
 from openstack_dashboard.api import glance
+from oslo_log import log as logging
 
 LOG = logging.getLogger(__name__)
 
@@ -32,8 +34,8 @@ def filter_murano_images(images, request=None):
             try:
                 metadata = json.loads(metadata)
             except ValueError:
-                msg = _('Invalid metadata for image: {0}'.format(image.id))
-                LOG.warn(msg)
+                msg = _('Invalid metadata for image: {0}').format(image.id)
+                LOG.warning(msg)
                 if request:
                     exceptions.handle(request, msg)
             else:
@@ -44,16 +46,31 @@ def filter_murano_images(images, request=None):
     return marked_images
 
 
-class MarkImageForm(SelfHandlingForm):
+class MarkImageForm(horizon_forms.SelfHandlingForm):
     _metadata = {
         'windows.2012': ' Windows Server 2012',
         'linux': 'Generic Linux',
-        'cirros.demo': 'Murano Demo'
+        'cirros.demo': 'CirrOS for Murano Demo',
+        'custom': "Custom type"
     }
 
-    image = forms.ChoiceField(label='Image')
+    image = forms.ChoiceField(label=_('Image'))
     title = forms.CharField(max_length="255", label=_("Title"))
-    type = forms.ChoiceField(label="Type", choices=_metadata.items())
+    type = forms.ChoiceField(
+        label=_("Type"),
+        choices=_metadata.items(),
+        initial='custom',
+        widget=forms.Select(attrs={
+            'class': 'switchable',
+            'data-slug': 'type'}))
+    custom_type = forms.CharField(
+        max_length="255",
+        label=_("Custom Type"),
+        widget=forms.TextInput(attrs={
+            'class': 'switched',
+            'data-switch-on': 'type',
+            'data-type-custom': _('Custom Type')}),
+        required=False)
     existing_titles = forms.CharField(widget=forms.HiddenInput)
 
     def __init__(self, request, *args, **kwargs):
@@ -61,11 +78,18 @@ class MarkImageForm(SelfHandlingForm):
 
         images = []
         try:
-            images, _more = glance.image_list_detailed(request)
+            # https://bugs.launchpad.net/murano/+bug/1339261 - glance
+            # client version change alters the API. Other tuple values
+            # are _more and _prev (in recent glance client)
+            images = glance.image_list_detailed(request)[0]
         except Exception:
             LOG.error('Failed to request image list from Glance')
             exceptions.handle(request, _('Unable to retrieve list of images'))
 
+        # filter out the image format aki and ari
+        images = filter(
+            lambda x: x.container_format not in ('aki', 'ari'), images)
+
         self.fields['image'].choices = [(i.id, i.name) for i in images]
         self.fields['existing_titles'].initial = \
             [image.title for image in filter_murano_images(images)]
@@ -74,27 +98,28 @@ class MarkImageForm(SelfHandlingForm):
         LOG.debug('Marking image with specified metadata: {0}'.format(data))
 
         image_id = data['image']
-        properties = {
-            'murano_image_info': json.dumps({
-                'title': data['title'],
-                'type': data['type']
-            })
-        }
-
+        image_type = data['type'] if data['type'] != 'custom' else \
+            data['custom_type']
+        kwargs = {}
+        kwargs['murano_image_info'] = json.dumps({
+            'title': data['title'],
+            'type': image_type
+        })
         try:
-            img = glance.image_update(request, image_id, properties=properties)
+            img = glance.image_update_properties(request, image_id, **kwargs)
             messages.success(request, _('Image successfully marked'))
             return img
         except Exception:
             exceptions.handle(request, _('Unable to mark image'),
-                              redirect='horizon:murano:images:index')
+                              redirect=reverse(
+                                  'horizon:app-catalog:images:index'))
 
     def clean_title(self):
         cleaned_data = super(MarkImageForm, self).clean()
         title = cleaned_data.get('title')
         existing_titles = self.fields['existing_titles'].initial
         if title in existing_titles:
-            raise ValidationError(_('Specified title already in use.'
-                                    ' Please choose another one.'))
+            raise forms.ValidationError(_('Specified title already in use.'
+                                          ' Please choose another one.'))
 
         return title
