@@ -14,15 +14,18 @@
 
 from django.core.urlresolvers import reverse
 from django.utils.translation import ugettext_lazy as _
+from django.utils.translation import ungettext_lazy
 from horizon import exceptions
 from horizon import tables
 from openstack_dashboard.api import glance
 
+from muranodashboard.common import utils as md_utils
+
 
 class MarkImage(tables.LinkAction):
     name = "mark_image"
     verbose_name = _("Mark Image")
-    url = "horizon:murano:images:mark_image"
+    url = "horizon:app-catalog:images:mark_image"
     classes = ("ajax-modal",)
     icon = "plus"
 
@@ -31,16 +34,30 @@ class MarkImage(tables.LinkAction):
 
 
 class RemoveImageMetadata(tables.DeleteAction):
-    data_type_singular = _('Metadata')
-    data_type_plural = _('Metadata')
+    @staticmethod
+    def action_present(count):
+        return ungettext_lazy(
+            u"Delete Metadata",
+            u"Delete Metadata",
+            count
+        )
+
+    @staticmethod
+    def action_past(count):
+        return ungettext_lazy(
+            u"Deleted Metadata",
+            u"Deleted Metadata",
+            count
+        )
 
     def delete(self, request, obj_id):
         try:
-            glance.image_update(request, obj_id,
-                                purge_props='murano_image_info')
+            remove_props = ['murano_image_info']
+            glance.image_update_properties(request, obj_id, remove_props)
         except Exception:
             exceptions.handle(request, _('Unable to remove metadata'),
-                              redirect=reverse('horizon:murano:images:index'))
+                              redirect=reverse(
+                                  'horizon:app-catalog:images:index'))
 
     def allowed(self, request, image):
         return request.user.is_superuser
@@ -54,13 +71,11 @@ class MarkedImagesTable(tables.DataTable):
     )
     type = tables.Column(lambda obj: getattr(obj, 'type', None),
                          verbose_name=_('Type'))
-    title = tables.Column(lambda obj: getattr(obj, 'title', None),
-                          verbose_name=_('Title'),
-                          truncate=40)
+    title = md_utils.Column(lambda obj: getattr(obj, 'title', None),
+                            verbose_name=_('Title'))
 
     class Meta(object):
         name = 'marked_images'
         verbose_name = _('Marked Images')
-        template = 'common/_data_table.html'
         table_actions = (MarkImage, RemoveImageMetadata)
         row_actions = (RemoveImageMetadata,)
