@@ -14,7 +14,6 @@
 
 import json
 import sys
-import urlparse
 
 from django.core.files import storage
 from django.core.urlresolvers import reverse
@@ -32,16 +31,18 @@ from horizon.forms import views
 from horizon import messages
 from horizon import tables as horizon_tables
 from horizon.utils import functions as utils
+from horizon import views as horizon_views
 from muranoclient.common import exceptions as exc
 from muranoclient.common import utils as muranoclient_utils
 from openstack_dashboard.api import glance
 from openstack_dashboard.api import keystone
 from oslo_log import log as logging
+import six
+import six.moves.urllib.parse as urlparse
 
 from muranodashboard import api
 from muranodashboard.api import packages as pkg_api
 from muranodashboard.catalog import views as catalog_views
-from muranodashboard import client_overrides
 from muranodashboard.common import utils as muranodashboard_utils
 from muranodashboard.environments import consts
 from muranodashboard.packages import consts as packages_consts
@@ -56,8 +57,6 @@ FORMS = [('upload', forms.ImportPackageForm),
 
 BUNDLE_FORMS = [('upload', forms.ImportBundleForm), ]
 
-client_overrides.override()
-
 
 def is_app(wizard):
     """Check if we're uploading an application
@@ -71,9 +70,65 @@ def is_app(wizard):
     return False
 
 
+def _ensure_images(name, package, request, step_data=None):
+    try:
+        glance_client = glance.glanceclient(
+            request, version='1')
+    except Exception:
+        glance_client = None
+
+    base_url = packages_consts.MURANO_REPO_URL
+    image_specs = package.images()
+    if not glance_client and len(image_specs):
+        # NOTE(kzaitsev): no glance_client. Probably v1 client
+        # is not available. Add warning, to let user know that
+        # we were unable to load images automagically
+        # since v2 does not have copy_from
+        download_urls = []
+        for image_spec in image_specs:
+            download_url = muranoclient_utils.to_url(
+                image_spec.get("Url", image_spec['Name']),
+                base_url=base_url,
+                path='images/',
+            )
+            download_urls.append(download_url)
+        msg = _("Couldn't initialise glance v1 client, "
+                "therefore could not download images for "
+                "'{0}' package. You may need to download them "
+                "manually from these locations: {1}").format(
+                    name, ' '.join(download_urls))
+        messages.error(request, msg)
+        LOG.error(msg)
+        return
+
+    try:
+        imgs = muranoclient_utils.ensure_images(
+            glance_client=glance_client,
+            image_specs=package.images(),
+            base_url=base_url)
+        for img in imgs:
+            msg = _("Trying to add {0} image to glance. "
+                    "Image will be ready for deployment after "
+                    "successful upload").format(img['name'],)
+            messages.warning(request, msg)
+            log_msg = _("Trying to add {0}, {1} image to "
+                        "glance. Image will be ready for "
+                        "deployment after successful upload")\
+                .format(img['name'], img['id'],)
+            LOG.info(log_msg)
+            if step_data:
+                step_data['images'].append(img)
+    except Exception as e:
+        msg = _("Error {0} occurred while installing "
+                "images for {1}").format(e, name)
+        messages.error(request, msg)
+        LOG.exception(msg)
+
+
 class PackageDefinitionsView(horizon_tables.DataTableView):
     table_class = tables.PackageDefinitionsTable
     template_name = 'packages/index.html'
+    page_title = _("Packages")
 
     _more = False
     _prev = False
@@ -142,7 +197,7 @@ class PackageDefinitionsView(horizon_tables.DataTableView):
                 if package.owner_id == current_tenant['id']:
                     packages[i].tenant_name = current_tenant['name']
                 else:
-                    packages[i].tenant_name = 'other'
+                    packages[i].tenant_name = _('UNKNOWN')
         return packages
 
     def get_context_data(self, **kwargs):
@@ -162,9 +217,10 @@ class PackageDefinitionsView(horizon_tables.DataTableView):
         return filters
 
 
-class ImportBundleWizard(views.ModalFormMixin,
+class ImportBundleWizard(horizon_views.PageTitleMixin, views.ModalFormMixin,
                          wizard_views.SessionWizardView):
     template_name = 'packages/import_bundle.html'
+    page_title = _("Import Bundle")
 
     def get_context_data(self, **kwargs):
         context = super(ImportBundleWizard, self).get_context_data(**kwargs)
@@ -216,12 +272,9 @@ class ImportBundleWizard(views.ModalFormMixin,
                 LOG.exception(msg)
                 messages.error(self.request, msg)
                 raise exceptions.Http302(
-                    reverse('horizon:murano:packages:index'))
-
-            glance_client = glance.glanceclient(self.request, version='1')
+                    reverse('horizon:app-catalog:packages:index'))
 
             for package_spec in bundle.package_specs():
-
                 try:
                     package = muranoclient_utils.Package.from_location(
                         package_spec['Name'],
@@ -238,23 +291,10 @@ class ImportBundleWizard(views.ModalFormMixin,
                     continue
 
                 reqs = package.requirements(base_url=base_url)
-                for dep_name, dep_package in reqs.iteritems():
-                    try:
-                        imgs = muranoclient_utils.ensure_images(
-                            glance_client=glance_client,
-                            image_specs=dep_package.images(),
-                            base_url=base_url)
-                        for img in imgs:
-                            msg = _("Added {0}, {1} image to glance").format(
-                                img['name'], img['id'],
-                            )
-                            messages.success(self.request, msg)
-                            LOG.info(msg)
-                    except Exception as e:
-                        msg = _("Error {0} occurred while installing "
-                                "images for {1}").format(e, dep_name)
-                        messages.error(self.request, msg)
-                        LOG.exception(msg)
+                for dep_name, dep_package in six.iteritems(reqs):
+                    _ensure_images(dep_name, dep_package,
+                                   self.request)
+
                     try:
                         files = {dep_name: dep_package.file()}
                         package = api.muranoclient(
@@ -289,18 +329,19 @@ class ImportBundleWizard(views.ModalFormMixin,
         return step_data
 
     def done(self, form_list, **kwargs):
-        redirect = reverse('horizon:murano:packages:index')
+        redirect = reverse('horizon:app-catalog:packages:index')
         msg = _('Bundle successfully imported.')
         LOG.info(msg)
         messages.success(self.request, msg)
         return http.HttpResponseRedirect(redirect)
 
 
-class ImportPackageWizard(views.ModalFormMixin,
+class ImportPackageWizard(horizon_views.PageTitleMixin, views.ModalFormMixin,
                           wizard_views.SessionWizardView):
     file_storage = storage.FileSystemStorage(location=consts.CACHE_DIR)
     template_name = 'packages/upload.html'
     condition_dict = {'add_category': is_app}
+    page_title = _("Import Package")
 
     def get_form_initial(self, step):
         initial_dict = self.initial_dict.get(step, {})
@@ -328,7 +369,10 @@ class ImportPackageWizard(views.ModalFormMixin,
         dep_pkgs = self.storage.get_step_data('upload').get(
             'dependencies', [])
 
-        redirect = reverse('horizon:murano:packages:index')
+        installed_images = self.storage.get_step_data('upload').get(
+            'images', [])
+
+        redirect = reverse('horizon:app-catalog:packages:index')
         dep_data = {'enabled': data['enabled'],
                     'is_public': data['is_public']}
         murano_client = api.muranoclient(self.request)
@@ -342,6 +386,33 @@ class ImportPackageWizard(views.ModalFormMixin,
                 LOG.warning(msg)
                 messages.warning(self.request, msg)
 
+        # Images have been imported as private images during the 'upload' step
+        # If the package is public, make the required images public
+        if data['is_public']:
+            try:
+                glance_client = glance.glanceclient(self.request, '1')
+            except Exception:
+                glance_client = None
+
+            if glance_client:
+                for img in installed_images:
+                    try:
+                        glance_client.images.update(img['id'], is_public=True)
+                        LOG.debug(
+                            'Success update for image {0}'.format(img['id']))
+                    except Exception as e:
+                        msg = _("Error {0} occurred while setting image {1}, "
+                                "{2} public").format(e, img['name'], img['id'])
+                        messages.error(self.request, msg)
+                        LOG.exception(msg)
+            elif len(installed_images):
+                msg = _("Couldn't initialise glance v1 client, "
+                        "therefore could not make the following images "
+                        "public: {0}").format(' '.join(
+                            [img['name'] for img in installed_images]))
+                messages.warning(self.request, msg)
+                LOG.warning(msg)
+
         try:
             data['tags'] = [t.strip() for t in data['tags'].split(',')]
             murano_client.packages.update(app_id, data)
@@ -351,7 +422,7 @@ class ImportPackageWizard(views.ModalFormMixin,
             LOG.exception(msg)
             exceptions.handle(
                 self.request, msg,
-                redirect=reverse('horizon:murano:packages:index'))
+                redirect=reverse('horizon:app-catalog:packages:index'))
         except (exc.HTTPException, Exception):
             LOG.exception(_('Modifying package failed'))
             exceptions.handle(self.request,
@@ -373,13 +444,15 @@ class ImportPackageWizard(views.ModalFormMixin,
                     reason = error.get('message')
             except ValueError:
                 # Let horizon operate with original exception
-                raise exc_info[0], exc_info[1], exc_info[2]
+                six.reraise(original_e.__class__,
+                            original_e.__class__(original_e),
+                            exc_info[2])
         msg = _('Uploading package failed. {0}').format(reason)
         LOG.exception(msg)
         exceptions.handle(
             self.request,
             msg,
-            redirect=reverse('horizon:murano:packages:index'))
+            redirect=reverse('horizon:app-catalog:packages:index'))
 
     def process_step(self, form):
         @catalog_views.update_latest_apps
@@ -422,32 +495,15 @@ class ImportPackageWizard(views.ModalFormMixin,
                 LOG.exception(msg)
                 messages.error(self.request, msg)
                 raise exceptions.Http302(
-                    reverse('horizon:murano:packages:index'))
-
-            def _ensure_images(name, package):
-                try:
-                    imgs = muranoclient_utils.ensure_images(
-                        glance_client=glance_client,
-                        image_specs=package.images(),
-                        base_url=base_url)
-                    for img in imgs:
-                        msg = _("Added {0}, {1} image to glance").format(
-                            img['name'], img['id'],
-                        )
-                        messages.success(self.request, msg)
-                        LOG.info(msg)
-                except Exception as e:
-                    msg = _("Error {0} occurred while installing "
-                            "images for {1}").format(e, name)
-                    messages.error(self.request, msg)
-                    LOG.exception(msg)
+                    reverse('horizon:app-catalog:packages:index'))
 
             reqs = package.requirements(base_url=base_url)
-            glance_client = glance.glanceclient(self.request, version='1')
             original_package = reqs.pop(name)
             step_data['dependencies'] = []
-            for dep_name, dep_package in reqs.iteritems():
-                _ensure_images(dep_name, dep_package)
+            step_data['images'] = []
+            for dep_name, dep_package in six.iteritems(reqs):
+                _ensure_images(dep_name, dep_package, self.request, step_data)
+
                 try:
                     files = {dep_name: dep_package.file()}
                     package = api.muranoclient(self.request).packages.create(
@@ -472,7 +528,7 @@ class ImportPackageWizard(views.ModalFormMixin,
                     continue
 
             # add main packages images
-            _ensure_images(name, original_package)
+            _ensure_images(name, original_package, self.request, step_data)
 
             # import main package itself
             try:
@@ -491,7 +547,7 @@ class ImportPackageWizard(views.ModalFormMixin,
                 exceptions.handle(
                     self.request,
                     msg,
-                    redirect=reverse('horizon:murano:packages:index'))
+                    redirect=reverse('horizon:app-catalog:packages:index'))
             except exc.HTTPInternalServerError as e:
                 self._handle_exception(e)
 
@@ -504,7 +560,7 @@ class ImportPackageWizard(views.ModalFormMixin,
                 exceptions.handle(
                     self.request,
                     reason,
-                    redirect=reverse('horizon:murano:packages:index'))
+                    redirect=reverse('horizon:app-catalog:packages:index'))
 
             except Exception as original_e:
                 self._handle_exception(original_e)
@@ -524,8 +580,9 @@ class ImportPackageWizard(views.ModalFormMixin,
 class ModifyPackageView(views.ModalFormView):
     form_class = forms.ModifyPackageForm
     template_name = 'packages/modify_package.html'
-    success_url = reverse_lazy('horizon:murano:packages:index')
-    failure_url = reverse_lazy('horizon:murano:packages:index')
+    success_url = reverse_lazy('horizon:app-catalog:packages:index')
+    failure_url = reverse_lazy('horizon:app-catalog:packages:index')
+    page_title = _("Modify Package")
 
     def get_initial(self):
         app_id = self.kwargs['app_id']
@@ -538,4 +595,46 @@ class ModifyPackageView(views.ModalFormView):
     def get_context_data(self, **kwargs):
         context = super(ModifyPackageView, self).get_context_data(**kwargs)
         context['app_id'] = self.kwargs['app_id']
+        context['type'] = kwargs['form'].initial['package'].type
         return context
+
+
+class DetailView(horizon_views.HorizonTemplateView):
+    template_name = 'packages/detail.html'
+    page_title = "{{ app.name }}"
+
+    def get_context_data(self, **kwargs):
+        context = super(DetailView, self).get_context_data(**kwargs)
+        app = self.get_data()
+        context["app"] = app
+        return context
+
+    def get_data(self):
+        app = None
+        try:
+            app_id = self.kwargs['app_id']
+            app = api.muranoclient(self.request).packages.get(app_id)
+        except Exception:
+            INDEX_URL = 'horizon:app-catalog:packages:index'
+            exceptions.handle(self.request,
+                              _('Unable to retrieve package details.'),
+                              redirect=reverse(INDEX_URL))
+        return app
+
+
+def download_packge(request, app_name, app_id):
+    try:
+        body = api.muranoclient(request).packages.download(app_id)
+
+        content_type = 'application/octet-stream'
+        response = http.HttpResponse(body, content_type=content_type)
+        response['Content-Disposition'] = 'filename={name}.zip'.format(
+            name=app_name)
+
+        return response
+    except exc.HTTPException:
+        LOG.exception(_('Something went wrong during package downloading'))
+        redirect = reverse('horizon:app-catalog:packages:index')
+        exceptions.handle(request,
+                          _('Unable to download package.'),
+                          redirect=redirect)
