@@ -10,24 +10,27 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+import contextlib
 import json
 import logging
 import os
-import sys
+import six.moves.urllib.parse as urlparse
 import testtools
 import time
-import urlparse
 import uuid
 
 from glanceclient import client as gclient
 from keystoneauth1.identity import v3
-from keystoneclient.v3 import client
-from muranoclient import client as mclient
+from keystoneauth1 import session as ks_session
+from keystoneclient.v3 import client as ks_client
+from muranoclient.common import exceptions as muranoclient_exc
+from muranoclient.glance import client as glare_client
+import muranoclient.v1.client as mclient
 from oslo_log import handlers
 from oslo_log import log
 from selenium.common import exceptions as exc
 from selenium import webdriver
-from selenium.webdriver.common.action_chains import ActionChains
+from selenium.webdriver.common import action_chains
 import selenium.webdriver.common.by as by
 from selenium.webdriver.support import expected_conditions as EC
 from selenium.webdriver.support import ui
@@ -40,19 +43,8 @@ logger = log.getLogger(__name__).logger
 logger.level = logging.DEBUG
 logger.addHandler(handlers.ColorHandler())
 
-if sys.version_info >= (2, 7):
-    class BaseDeps(testtools.TestCase):
-        pass
-else:
-    # Define asserts for python26
-    import unittest2
 
-    class BaseDeps(testtools.TestCase,
-                   unittest2.TestCase):
-        pass
-
-
-class UITestCase(BaseDeps):
+class UITestCase(testtools.TestCase):
     @classmethod
     def setUpClass(cls):
         auth = v3.Password(user_domain_name='Default',
@@ -61,12 +53,23 @@ class UITestCase(BaseDeps):
                            project_domain_name='Default',
                            project_name=cfg.common.tenant,
                            auth_url=cfg.common.keystone_url)
-        cls.keystone_client = client.Client(
-            auth_url=cfg.common.keystone_url, auth=auth,
-            username=cfg.common.user, password=cfg.common.password)
+        session = ks_session.Session(auth=auth)
+        cls.keystone_client = ks_client.Client(session=session)
+        cls.auth_ref = auth.get_auth_ref(session)
+        cls.service_catalog = cls.auth_ref.service_catalog
+        if utils.glare_enabled():
+            glare_endpoint = "http://127.0.0.1:9494"
+            artifacts_client = glare_client.Client(
+                endpoint=glare_endpoint, token=cls.auth_ref.auth_token,
+                insecure=False, key_file=None, ca_file=None, cert_file=None,
+                type_name="murano", type_version=1)
+        else:
+            artifacts_client = None
         cls.murano_client = mclient.Client(
-            '1', endpoint=cfg.common.murano_url,
-            token=cls.keystone_client.auth_token)
+            artifacts_client=artifacts_client,
+            endpoint_override=cfg.common.murano_url,
+            session=session)
+
         cls.url_prefix = urlparse.urlparse(cfg.common.horizon_url).path or ''
         if cls.url_prefix.endswith('/'):
             cls.url_prefix = cls.url_prefix[:-1]
@@ -74,10 +77,19 @@ class UITestCase(BaseDeps):
     def setUp(self):
         super(UITestCase, self).setUp()
 
-        self.driver = webdriver.Firefox()
+        # Enables zip files to be automatically saved to disk, without opening
+        # a browser dialog.
+        fp = webdriver.FirefoxProfile()
+        fp.set_preference("browser.download.folderList", 2)
+        fp.set_preference("browser.download.manager.showWhenStarting", False)
+        fp.set_preference("browser.download.dir", os.getcwd())
+        fp.set_preference("browser.helperApps.neverAsk.saveToDisk",
+                          "application/octet-stream")
+
+        self.driver = webdriver.Firefox(firefox_profile=fp)
         self.addCleanup(self.driver.quit)
         self.driver.maximize_window()
-        self.driver.get(cfg.common.horizon_url + '/murano/environments')
+        self.driver.get(cfg.common.horizon_url + '/app-catalog/environments')
         self.driver.implicitly_wait(30)
         self.addOnException(self.take_screenshot)
         self.log_in()
@@ -158,13 +170,15 @@ class UITestCase(BaseDeps):
         self.projects_to_delete.append(project.id)
         return project.id
 
-    def add_user_to_project(self, project_id, user_name, user_role=None):
+    def add_user_to_project(self, project_id, user_id, user_role=None):
         if not user_role:
             roles = self.keystone_client.roles.list()
             role_id = [role.id for role in roles if role.name == 'Member'][0]
-        if not user_name:
+        if not user_id:
             user_name = cfg.common.user
-        self.keystone_client.roles.grant(role_id, user=user_name,
+            users = self.keystone_client.users.list()
+            user_id = [user.id for user in users if user.name == user_name][0]
+        self.keystone_client.roles.grant(role_id, user=user_id,
                                          project=project_id)
 
     def switch_to_project(self, name):
@@ -203,7 +217,7 @@ class UITestCase(BaseDeps):
         self.fill_field(by.By.ID, 'id_username', username)
         self.fill_field(by.By.ID, 'id_password', password)
         self.driver.find_element_by_xpath("//button[@type='submit']").click()
-        murano = self.driver.find_element_by_xpath(consts.Murano)
+        murano = self.driver.find_element_by_xpath(consts.AppCatalog)
         if 'collapsed' in murano.get_attribute('class'):
             murano.click()
 
@@ -218,18 +232,22 @@ class UITestCase(BaseDeps):
         self.driver.find_element(by=by_find, value=field).clear()
         self.driver.find_element(by=by_find, value=field).send_keys(value)
 
-    def get_element_id(self, el_name):
-        el = ui.WebDriverWait(self.driver, 10).until(
+    def get_element_id(self, el_name, sec=10):
+        el = ui.WebDriverWait(self.driver, sec).until(
             EC.presence_of_element_located(
                 (by.By.XPATH, consts.AppPackages.format(el_name))))
         path = el.get_attribute("id")
         return path.split('__')[-1]
 
     def select_and_click_action_for_app(self, action, app):
-        self.driver.find_element_by_xpath(
-            "//*[@href='{0}/murano/catalog/{1}/{2}']".format(self.url_prefix,
-                                                             action,
-                                                             app)).click()
+        self.check_element_on_page(
+            by.By.XPATH,
+            "//*[@href='{0}/app-catalog/catalog/{1}/{2}']".format(
+                self.url_prefix, action, app))
+        self.wait_element_is_clickable(
+            by.By.XPATH,
+            "//*[@href='{0}/app-catalog/catalog/{1}/{2}']".format(
+                self.url_prefix, action, app)).click()
 
     def go_to_submenu(self, link):
         element = self.wait_element_is_clickable(by.By.PARTIAL_LINK_TEXT, link)
@@ -242,18 +260,16 @@ class UITestCase(BaseDeps):
                           ".//*[@class='page-header']").text)
 
     def navigate_to(self, menu):
-        el = ui.WebDriverWait(self.driver, 10).until(
-            EC.presence_of_element_located(
-                (by.By.XPATH, getattr(consts, menu))))
+        el = self.wait_element_is_clickable(
+            by.By.XPATH, getattr(consts, menu))
         if 'collapsed' in el.get_attribute('class'):
             el.click()
-        self.wait_for_sidebar_is_loaded()
 
-    def select_from_list(self, list_name, value):
+    def select_from_list(self, list_name, value, sec=10):
         locator = (by.By.XPATH,
                    "//select[contains(@name, '{0}')]"
                    "/option[@value='{1}']".format(list_name, value))
-        el = ui.WebDriverWait(self.driver, 10).until(
+        el = ui.WebDriverWait(self.driver, sec).until(
             EC.presence_of_element_located(locator))
         el.click()
 
@@ -262,16 +278,16 @@ class UITestCase(BaseDeps):
             ui.WebDriverWait(self.driver, sec).until(
                 EC.presence_of_element_located((method, value)))
         except exc.TimeoutException:
-            self.fail("Element {0} is not preset on the page".format(value))
+            self.fail("Element {0} is not present on the page".format(value))
 
-    def check_element_not_on_page(self, method, value):
-        self.driver.implicitly_wait(3)
+    def check_element_not_on_page(self, method, value, sec=3):
+        self.driver.implicitly_wait(sec)
         present = True
         try:
             self.driver.find_element(method, value)
         except (exc.NoSuchElementException, exc.ElementNotVisibleException):
             present = False
-        self.assertFalse(present, u"Element {0} is preset on the page"
+        self.assertFalse(present, u"Element {0} is present on the page"
                                   " while it should't".format(value))
         self.driver.implicitly_wait(30)
 
@@ -281,7 +297,7 @@ class UITestCase(BaseDeps):
             ui.WebDriverWait(self.driver, sec).until(
                 EC.presence_of_element_located(locator))
         except exc.TimeoutException:
-            self.fail("Alert is not preset on the page")
+            self.fail("Alert is not present on the page")
 
         self.assertIn(message, self.driver.find_element(*locator).text)
 
@@ -296,8 +312,9 @@ class UITestCase(BaseDeps):
         self.driver.find_element_by_id(consts.ConfirmCreateEnvironment).click()
         self.wait_for_alert_message()
 
-    def delete_environment(self, env_name):
-        self.select_action_for_environment(env_name, 'delete')
+    def delete_environment(self, env_name, from_detail_view=False):
+        if not from_detail_view:
+            self.select_action_for_environment(env_name, 'delete')
         self.driver.find_element_by_xpath(consts.ConfirmDeletion).click()
         self.wait_for_alert_message()
 
@@ -309,7 +326,7 @@ class UITestCase(BaseDeps):
             'button.ajax-inline-edit')
 
         # hover to make pencil visible
-        hover = ActionChains(self.driver).move_to_element(el_td)
+        hover = action_chains.ActionChains(self.driver).move_to_element(el_td)
         hover.perform()
         el_pencil.click()
 
@@ -335,12 +352,21 @@ class UITestCase(BaseDeps):
         btn_id = "environments__row_{0}__action_{1}".format(element_id, action)
         self.driver.find_element_by_id(btn_id).click()
 
-    def wait_for_alert_message(self):
+    def wait_for_alert_message(self, sec=5):
         locator = (by.By.CSS_SELECTOR, 'div.alert-success')
         logger.debug("Waiting for a success message")
-        ui.WebDriverWait(self.driver, 5).until(
+        ui.WebDriverWait(self.driver, sec).until(
             EC.presence_of_element_located(locator))
 
+    def wait_for_alert_message_to_disappear(self, sec=10):
+        # The alert message pops up directly over the delete environment
+        # button, causing click issues. So we must wait for the alert message
+        # to completely disappear before clicking the button.
+        locator = (by.By.CSS_SELECTOR, 'div.alert-success')
+        logger.debug("Waiting for a success message to disappear")
+        ui.WebDriverWait(self.driver, sec).until(
+            EC.invisibility_of_element_located(locator))
+
     def wait_for_error_message(self, sec=20):
         locator = (by.By.CSS_SELECTOR, 'div.alert-danger > p')
         logger.debug("Waiting for an error message")
@@ -348,50 +374,69 @@ class UITestCase(BaseDeps):
             EC.presence_of_element_located(locator))
         return self.driver.find_element(*locator).text
 
-    def wait_element_is_clickable(self, method, element):
-        return ui.WebDriverWait(self.driver, 10).until(
+    def wait_element_is_clickable(self, method, element, sec=10):
+        return ui.WebDriverWait(self.driver, sec).until(
             EC.element_to_be_clickable((method, element)))
 
-    def wait_for_sidebar_is_loaded(self):
-        ui.WebDriverWait(self.driver, 10).until(
+    def wait_for_sidebar_is_loaded(self, sec=10):
+        ui.WebDriverWait(self.driver, sec).until(
             EC.presence_of_element_located(
-                (by.By.CSS_SELECTOR, "div#sidebar li.active")))
+                (by.By.CSS_SELECTOR, "nav#sidebar a.active")))
         time.sleep(0.5)
 
+    @contextlib.contextmanager
+    def wait_for_page_reload(self, sec=10):
+        old_page = self.driver.find_element_by_tag_name('html')
+        yield
+        ui.WebDriverWait(self, sec).until(EC.staleness_of(old_page))
+
 
 class PackageBase(UITestCase):
     @classmethod
     def setUpClass(cls):
         super(PackageBase, cls).setUpClass()
-        cls.mockapp_id = utils.upload_app_package(
-            cls.murano_client,
+        cls.packages = []
+        cls.mockapp_id = cls.upload_package(
             "MockApp",
             {"categories": ["Web"], "tags": ["tag"]})
-        cls.postgre_id = utils.upload_app_package(
-            cls.murano_client,
+        cls.postgre_id = cls.upload_package(
             "PostgreSQL",
             {"categories": ["Databases"], "tags": ["tag"]})
-        cls.hot_app_id = utils.upload_app_package(
-            cls.murano_client,
+        cls.hot_app_id = cls.upload_package(
             "HotExample",
             {"tags": ["hot"]}, hot=True)
+        cls.deployingapp_id = cls.upload_package(
+            "DeployingApp",
+            {"categories": ["Web"], "tags": ["tag"]},
+            hot=False,
+            package_dir=consts.DeployingPackageDir)
+
+    @classmethod
+    def upload_package(cls, name, data, **kwargs):
+        package = utils.upload_app_package(cls.murano_client, name, data,
+                                           **kwargs)
+        cls.packages.append(package)
+        return package
 
     @classmethod
     def tearDownClass(cls):
         super(PackageBase, cls).tearDownClass()
-        cls.murano_client.packages.delete(cls.mockapp_id)
-        cls.murano_client.packages.delete(cls.postgre_id)
-        cls.murano_client.packages.delete(cls.hot_app_id)
+        # In case dynamically created packages are deleted at test level,
+        # ignore not found errors below.
+        for package in cls.packages:
+            try:
+                cls.murano_client.packages.delete(package)
+            except muranoclient_exc.HTTPNotFound:
+                pass
 
 
 class ImageTestCase(PackageBase):
     @classmethod
     def setUpClass(cls):
         super(ImageTestCase, cls).setUpClass()
-        glance_endpoint = cls.keystone_client.service_catalog.url_for(
-            service_type='image', endpoint_type='publicURL')
+        glance_endpoint = cls.service_catalog.url_for(service_type='image')
         cls.glance = gclient.Client('1', endpoint=glance_endpoint,
-                                    token=cls.keystone_client.auth_token)
+                                    session=cls.keystone_client.session)
         cls.image_title = 'New Image ' + str(time.time())
         cls.image = cls.upload_image(cls.image_title)
 
@@ -405,15 +450,14 @@ class ImageTestCase(PackageBase):
         try:
             property = {'murano_image_info': json.dumps({'title': title,
                                                          'type': 'linux'})}
-
             image = cls.glance.images.create(name='TestImage',
                                              disk_format='qcow2',
                                              size=0,
                                              is_public=True,
                                              properties=property)
-        except Exception as e:
+        except Exception:
             logger.error("Unable to create or update image in Glance")
-            raise e
+            raise
         return image
 
     def select_and_click_element(self, element):
@@ -452,22 +496,28 @@ class FieldsTestCase(PackageBase):
 
 
 class ApplicationTestCase(ImageTestCase):
-    def delete_component(self, component_name):
-        component_id = self.get_element_id(component_name)
-        self.driver.find_element_by_id(
-            'services__row_{0}__action_delete'.format(component_id)).click()
+    def delete_component(self, component_name=None):
+        if component_name:
+            component_id = self.get_element_id(component_name)
+            btn = self.driver.find_element_by_id(
+                'services__row_{0}__action_delete'.format(
+                    component_id))
+        else:
+            btn = self.driver.find_element_by_css_selector(
+                consts.DeleteComponent)
+        btn.click()
         el = self.wait_element_is_clickable(by.By.LINK_TEXT,
-                                            'Delete Component')
+                                            'Delete Component', sec=30)
         el.click()
         self.wait_for_alert_message()
 
-    def select_action_for_package(self, package_id, action):
+    def select_action_for_package(self, package_id, action, sec=10):
         if action == 'more':
             el = self.wait_element_is_clickable(
                 by.By.XPATH, "//tr[@data-object-id='{0}']"
                              "//a[@data-toggle='dropdown']".format(package_id))
             el.click()
-            ui.WebDriverWait(self.driver, 10).until(lambda s: s.find_element(
+            ui.WebDriverWait(self.driver, sec).until(lambda s: s.find_element(
                 by.By.XPATH,
                 ".//*[@id='packages__row_{0}__action_download_package']".
                 format(package_id)).is_displayed())
@@ -498,9 +548,16 @@ class ApplicationTestCase(ImageTestCase):
         self.driver.find_element_by_xpath(consts.InputSubmit).click()
         self.wait_for_alert_message()
 
-    def add_app_to_env(self, app_id, app_name='TestApp'):
-        self.go_to_submenu('Applications')
-        self.select_and_click_action_for_app('quick-add', app_id)
+    def add_app_to_env(self, app_id, app_name='TestApp', env_id=None):
+        self.navigate_to('Browse')
+        self.go_to_submenu('Browse Local')
+        if env_id:
+            action = 'add'
+            app = '{0}/{1}'.format(app_id, env_id)
+        else:
+            action = 'quick-add'
+            app = app_id
+        self.select_and_click_action_for_app(action, app)
         field_id = "{0}_0-name".format(app_id)
         self.fill_field(by.By.ID, field_id, value=app_name)
         self.driver.find_element_by_xpath(consts.ButtonSubmit).click()
@@ -508,7 +565,37 @@ class ApplicationTestCase(ImageTestCase):
         self.select_from_list('osImage', self.image.id)
 
         self.driver.find_element_by_xpath(consts.InputSubmit).click()
-        self.wait_for_alert_message()
+        if env_id:
+            self.driver.find_element_by_xpath(consts.InputSubmit).click()
+            self.wait_element_is_clickable(by.By.ID, consts.AddComponent)
+            self.check_element_on_page(by.By.LINK_TEXT, app_name)
+        else:
+            self.wait_for_alert_message()
+
+    def execute_action_from_table_view(self, env_name, table_action):
+        """Executes an action like Deploy or Delete from the table view.
+
+        Does not handle clicking on the confirmation modal that may appear.
+
+        Scenario:
+            1. Checks for the table drop-down button and then clicks it.
+            2. Checks for the table drop-down menu to appear.
+            3. Checks for the ``table_action`` button and then clicks it.
+        """
+        self.check_element_on_page(
+            by.By.XPATH, consts.TableDropdownBtn.format(env_name))
+        dropdown_btn = self.driver.find_element(
+            by.By.XPATH, consts.TableDropdownBtn.format(env_name))
+        dropdown_btn.click()
+        self.check_element_on_page(by.By.XPATH,
+                                   consts.TableDropdownMenu.format(env_name))
+        self.check_element_on_page(
+            by.By.XPATH, consts.TableDropdownAction.format(env_name,
+                                                           table_action))
+        action_btn = self.driver.find_element(
+            by.By.XPATH, consts.TableDropdownAction.format(env_name,
+                                                           table_action))
+        action_btn.click()
 
 
 class PackageTestCase(ApplicationTestCase):
